const ESLintPluginImportTypeScript = require('eslint-plugin-import/config/typescript')
const ESLintConfigPrettierTypeScript = require('eslint-config-prettier/@typescript-eslint')

module.exports = {
  root: true,
  parser: 'babel-eslint',
  extends: [
    'eslint-config-standard',
    './eslint-plugin-import-dollarshaveclub',
    'eslint-config-prettier',
    'eslint-config-prettier/babel',
    'eslint-config-prettier/standard',
  ].map(require.resolve),
  plugins: ['prettier'],
  overrides: [
    {
      files: ['**/*.ts', '**/*.tsx'],
      parser: '@typescript-eslint/parser',
      parserOptions: {
        ecmaVersion: 2018,
        sourceType: 'module',
        ecmaFeatures: {
          jsx: true,
        },

        // typescript-eslint specific options
        warnOnUnsupportedTypeScriptVersion: true,
      },
      plugins: ['@typescript-eslint'],
      settings: {
        // Currently ESLint doesn't support extending in overrides
        // To work around this, you can require the overrides of the desired sub config directly
        ...ESLintPluginImportTypeScript.settings,
        'import/resolver': {
          ...ESLintPluginImportTypeScript.settings['import/resolver'],
          // use <root>/tsconfig.json
          typescript: {},
        },
      },
      rules: {
        // These ESLint rules are known to cause issues with typescript-eslint
        // See https://github.com/typescript-eslint/typescript-eslint/blob/master/packages/eslint-plugin/src/configs/recommended.json
        'camelcase': 'off',
        'indent': 'off',
        'no-array-constructor': 'off',
        'no-unused-vars': 'off',

        '@typescript-eslint/no-array-constructor': 'warn',
        '@typescript-eslint/no-namespace': 'error',
        '@typescript-eslint/no-unused-vars': [
          'error',
          {
            args: 'none',
            ignoreRestSiblings: true,
          },
        ],
        ...ESLintConfigPrettierTypeScript.rules,
      },
    },
    {
      files: ['**/*.test.ts', '**/*.test.tsx', '**/__{mocks,tests}__/**'],
      plugins: ['jest'],
      env: {
        jest: true,
      },
      rules: {
        ...require('eslint-plugin-jest').configs.recommended.rules,
      },
    },
  ],
  rules: {
    'no-plusplus': 'off',
    'no-restricted-syntax': 'off',
    'prefer-const': [
      'error',
      { destructuring: 'any', ignoreReadBeforeAssign: true },
    ],
    'comma-dangle': ['error', 'always-multiline'],
    'camelcase': 'off',
    'func-names': 'warn',
    'no-await-in-loop': 'off',
    'no-multi-assign': 'off',
    'no-underscore-dangle': 'off',
    'quotes': [
      'error',
      'single',
      { avoidEscape: true, allowTemplateLiterals: true },
    ],
    'no-console': ['error', { allow: ['warn', 'error'] }],

    // Prettier!
    'prettier/prettier': 'error',
  },
}
